/**
 * Title :        XLogo
 * Description :  XLogo is an interpreter for the Logo 
 * 						programming language
 * @author Loïc Le Coq
 */
package xlogo;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;

import javax.swing.*;

import java.util.Stack;
import java.io.*;

import xlogo.utils.Utils;
import xlogo.gui.preferences.Panel_Font;
import xlogo.gui.preferences.Preference;
import xlogo.gui.*;
import xlogo.kernel.Affichage;
import xlogo.kernel.Kernel;

public class Application extends JFrame {
	private static Stack pile_historique = new Stack();
	private int index_historique = 0;

	// Contient le chemin du fichiers actuellement
	// ouvert
	public static String path = null;
	public String tmp_path = null; // Lorsqu'on ouvre un fichier
	
	//	 numéro identifiant de la police de l'interface
	public static int police = Panel_Font.police_id(Config.police); 
  	
	private MenuListener menulistener;
	public Editor editeur;
	
	private JPanel contentPane;
	private BorderLayout borderLayout1 = new BorderLayout();
	private JPanel jPanel1 = new JPanel();
	private ZoneCommande commande=new ZoneCommande(this);
	private JLabel jLabel1 = new JLabel();
	private BorderLayout borderLayout2 = new BorderLayout();
	public JSplitPane jSplitPane1 = new JSplitPane();
	public JScrollPane jScrollPane1;
	private Ardoise ardoise1; 
	private HistoryPanel panneauHistorique1 = new HistoryPanel(this);

	public boolean error = false;
	boolean stop = false;

	public Affichage affichage = null;

	private Sound_Player son = new Sound_Player(this);
	private Touche touche = new Touche();
	
	
	private JMenuBar jMenuBar1 = new JMenuBar();
	private JMenu jMenuFile = new JMenu();
	private JMenuItem jMenuFileNouveau = new JMenuItem();
	private JMenuItem jMenuFileEnregistrer = new JMenuItem();
	private JMenuItem jMenuFileEnregistrer_sous = new JMenuItem();
	private JMenuItem jMenuFileOuvrir = new JMenuItem();
	private JMenuItem jMenuFileQuitter = new JMenuItem();
	private JMenu jMenuCapture = new JMenu();
	private JMenuItem jMenuImageCopy = new JMenuItem();
	private JMenuItem jMenuImageSave = new JMenuItem();
	private JMenuItem jMenuImagePrint = new JMenuItem();
	private JMenu jMenuZoneTexte=new JMenu();
	private JMenuItem jMenuZoneTexteItem = new JMenuItem();

	private JMenu jMenuEdition = new JMenu();
	private JMenuItem jMenuEditionCopier = new JMenuItem();
	private JMenuItem jMenuEditionColler = new JMenuItem();
	private JMenuItem jMenuEditionCouper = new JMenuItem();
	private JMenuItem jMenuEditionSelect = new JMenuItem();
	
	private JMenu jMenuOptions = new JMenu();
	private JMenuItem jMenuOptionsCouleurCrayon = new JMenuItem();
	private JMenuItem jMenuOptionsCouleurFond = new JMenuItem();
	private JMenuItem jMenuOptionsDemarrage = new JMenuItem();
	private JMenuItem jMenuOptionsPreference = new JMenuItem();
	private JMenuItem jMenuOptionsTraduction = new JMenuItem();
	private JMenuItem jMenuOptionsEraser = new JMenuItem();
	private Popup jpop;

	private JMenu jMenuHelp = new JMenu();
	private JMenuItem jMenuHelpAbout = new JMenuItem();
	private JMenuItem jMenuHelpLicence = new JMenuItem();
	private JMenuItem jMenuHelplicencefrancais = new JMenuItem();

	// Dialog boxes available in menu
	// pref Box
	private Preference pf=null;
	// To save image in the clipboard or in a file
	private Dialogueimage diag=null;
	// To define startup files
	private Demarrage dem=null;
	// To erase procedure
	private ProcedureEraser eraser=null;
	// To translate files
	private Traduc traduc=null;

	
	// Interpreter and drawer
	private Kernel kernel;
	
	/** Construire le cadre */
	public Application() {
		jScrollPane1= new JScrollPane();
		ardoise1= new Ardoise(jScrollPane1);
		kernel=new Kernel(this);
		kernel.initInterprete();
		menulistener= new MenuListener(this);
		jpop=new Popup(menulistener, commande);
		editeur=new Editor(this);		 
		enableEvents(AWTEvent.WINDOW_EVENT_MASK);
		try {
			initGui();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	/** Initialiser le composant */
	private void initGui() throws Exception {
		this.setText();
		setIconImage(Toolkit.getDefaultToolkit().createImage(
		Utils.class.getResource("icone.png")));
		contentPane = (JPanel) this.getContentPane();
		contentPane.setLayout(borderLayout1);
		Dimension d=Toolkit.getDefaultToolkit().getScreenSize();
		this.setSize(new Dimension(d.width,d.height*9/10));
		this.setTitle("XLogo");
		
		jMenuHelpLicence.addActionListener(menulistener);
		jMenuHelpLicence.setActionCommand(MenuListener.HELP_LICENCE);
		jMenuHelpAbout.addActionListener(menulistener);
		jMenuHelpAbout.setActionCommand(MenuListener.HELP_ABOUT);
		jMenuHelplicencefrancais.addActionListener(menulistener);
		jMenuHelplicencefrancais.setActionCommand(MenuListener.HELP_TRANSLATED_LICENCE);
		
		
		/*commande.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				commande_actionPerformed(e);
			}
		});*/
		commande.setPreferredSize(new Dimension(300,18*Config.police.getSize()/10));
		jPanel1.setLayout(borderLayout2);
		jSplitPane1.setOrientation(JSplitPane.VERTICAL_SPLIT);
		panneauHistorique1.setMinimumSize(new Dimension(600, 40));

		jMenuEditionCopier.addActionListener(menulistener);
		jMenuEditionCopier.setActionCommand(MenuListener.EDIT_COPY);
		jMenuEditionColler.addActionListener(menulistener);
		jMenuEditionColler.setActionCommand(MenuListener.EDIT_PASTE);
		jMenuEditionCouper.addActionListener(menulistener);
		jMenuEditionCouper.setActionCommand(MenuListener.EDIT_CUT);
		jMenuEditionSelect.addActionListener(menulistener);
		jMenuEditionSelect.setActionCommand(MenuListener.EDIT_SELECT_ALL);
		
		
		jMenuFileOuvrir.addActionListener(menulistener);
		jMenuFileOuvrir.setActionCommand(MenuListener.FILE_OPEN);
		jMenuFileNouveau.addActionListener(menulistener);
		jMenuFileNouveau.setActionCommand(MenuListener.FILE_NEW);
		jMenuFileNouveau.setEnabled(false);
		jMenuFileEnregistrer_sous.addActionListener(menulistener);
		jMenuFileEnregistrer_sous.setActionCommand(MenuListener.FILE_SAVE_AS);
		jMenuFileEnregistrer.addActionListener(menulistener);
		jMenuFileEnregistrer.setActionCommand(MenuListener.FILE_SAVE);
		jMenuFileEnregistrer.setEnabled(false);
		jMenuFileQuitter.addActionListener(menulistener);
		jMenuFileQuitter.setActionCommand(MenuListener.FILE_QUIT);

		jMenuImageCopy.addActionListener(menulistener);
		jMenuImageCopy.setActionCommand(MenuListener.FILE_COPY_IMAGE);
		jMenuImageSave.addActionListener(menulistener);
		jMenuImageSave.setActionCommand(MenuListener.FILE_SAVE_IMAGE);
		jMenuImagePrint.addActionListener(menulistener);
		jMenuImagePrint.setActionCommand(MenuListener.FILE_PRINT_IMAGE);
		jMenuZoneTexteItem.addActionListener(menulistener);
		jMenuZoneTexteItem.setActionCommand(MenuListener.FILE_SAVE_TEXT);

		jMenuOptionsDemarrage.addActionListener(menulistener);
		jMenuOptionsDemarrage.setActionCommand(MenuListener.TOOLS_START_FILE);
		jMenuOptionsCouleurCrayon.addActionListener(menulistener);
		jMenuOptionsCouleurCrayon.setActionCommand(MenuListener.TOOLS_PEN_COLOR);
		jMenuOptionsCouleurFond.addActionListener(menulistener);
		jMenuOptionsCouleurFond.setActionCommand(MenuListener.TOOLS_SCREEN_COLOR);
		jMenuOptionsPreference.addActionListener(menulistener);
		jMenuOptionsPreference.setActionCommand(MenuListener.TOOLS_OPTIONS);
		jMenuOptionsTraduction.addActionListener(menulistener);
		jMenuOptionsTraduction.setActionCommand(MenuListener.TOOLS_TRANSLATOR);
		jMenuOptionsEraser.addActionListener(menulistener);
		jMenuOptionsEraser.setActionCommand(MenuListener.TOOLS_ERASER);
		jMenuFile.add(jMenuFileNouveau);
		jMenuFile.add(jMenuFileOuvrir);
		jMenuFile.add(jMenuFileEnregistrer_sous);
		jMenuFile.add(jMenuFileEnregistrer);
		jMenuFile.add(jMenuCapture);
		jMenuZoneTexte.add(jMenuZoneTexteItem);
		jMenuFile.add(jMenuZoneTexte);
		jMenuFile.add(jMenuFileQuitter);
		jMenuHelp.add(jMenuHelpLicence);
		jMenuHelp.add(jMenuHelplicencefrancais);
		jMenuHelp.add(jMenuHelpAbout);
		jMenuBar1.add(jMenuFile);
		jMenuBar1.add(jMenuEdition);
		jMenuBar1.add(jMenuOptions);
		jMenuBar1.add(jMenuHelp);
		contentPane.add(jPanel1, BorderLayout.NORTH);
		jPanel1.add(jLabel1, BorderLayout.WEST);
		jPanel1.add(commande, BorderLayout.CENTER);
		contentPane.add(jSplitPane1, BorderLayout.CENTER);
		jSplitPane1.add(jScrollPane1, JSplitPane.LEFT);
		
		jScrollPane1.getViewport().add(ardoise1);
		jScrollPane1.getHorizontalScrollBar().setBlockIncrement(5);
		jScrollPane1.getVerticalScrollBar().setBlockIncrement(5);
		
		jSplitPane1.add(panneauHistorique1, JSplitPane.RIGHT);
		jMenuEdition.add(jMenuEditionCouper);
		jMenuEdition.add(jMenuEditionCopier);
		jMenuEdition.add(jMenuEditionColler);
		jMenuEdition.add(jMenuEditionSelect);
		jMenuCapture.add(jMenuImageCopy);
		jMenuCapture.add(jMenuImageSave);
		jMenuCapture.add(jMenuImagePrint);
		jMenuOptions.add(jMenuOptionsCouleurCrayon);
		jMenuOptions.add(jMenuOptionsCouleurFond);
		jMenuOptions.add(jMenuOptionsDemarrage);
		jMenuOptions.add(jMenuOptionsTraduction);
		jMenuOptions.add(jMenuOptionsEraser);
		jMenuOptions.add(jMenuOptionsPreference);

		MouseListener popupListener = new PopupListener();
		commande.addMouseListener(popupListener);

		this.setJMenuBar(jMenuBar1);
		jSplitPane1.setResizeWeight(0.8);
		commande.addKeyListener(touche);
	}

	public void setText() {
		//		System.out.println(Config.police.getName());
		setFont(Config.police);
		// Texte interne à utiliser pour JFileChooser et JColorChooser
		UIManager.put("FileChooser.cancelButtonText", Logo.messages
				.getString("annuler"));
		UIManager.put("FileChooser.cancelButtonToolTipText", Logo.messages
				.getString("annuler"));
		UIManager.put("FileChooser.fileNameLabelText", Logo.messages
				.getString("nom_du_fichier"));
		UIManager.put("FileChooser.upFolderToolTipText", Logo.messages
				.getString("dossier_parent"));
		UIManager.put("FileChooser.lookInLabelText", Logo.messages
				.getString("rechercher_dans"));

		UIManager.put("FileChooser.newFolderToolTipText", Logo.messages
				.getString("nouveau_dossier"));
		UIManager.put("FileChooser.homeFolderToolTipText", Logo.messages
				.getString("repertoire_accueil"));
		UIManager.put("FileChooser.filesOfTypeLabelText", Logo.messages
				.getString("fichier_du_type"));
		UIManager.put("FileChooser.helpButtonText", Logo.messages
				.getString("aide"));

		UIManager.put("ColorChooser.rgbNameText", Logo.messages
				.getString("rgb"));
		UIManager.put("ColorChooser.rgbBlueText", Logo.messages
				.getString("bleu"));
		UIManager.put("ColorChooser.rgbGreenText", Logo.messages
				.getString("vert"));
		UIManager.put("ColorChooser.rgbRedText", Logo.messages
				.getString("rouge"));

		UIManager.put("ColorChooser.swatchesNameText", Logo.messages
				.getString("echantillon"));

		UIManager.put("ColorChooser.hsbNameText", Logo.messages
				.getString("hsb"));
		UIManager.put("ColorChooser.hsbBlueText", Logo.messages
				.getString("hsbbleu"));
		UIManager.put("ColorChooser.hsbGreenText", Logo.messages
				.getString("hsbvert"));
		UIManager.put("ColorChooser.hsbRedText", Logo.messages
				.getString("hsbrouge"));

		UIManager.put("ColorChooser.swatchesRecentText", Logo.messages
				.getString("dernier"));
		UIManager.put("ColorChooser.previewText", Logo.messages
				.getString("apercu"));
		UIManager.put("ColorChooser.sampleText", Logo.messages
				.getString("echantillon_texte"));
		UIManager.put("ColorChooser.okText", Logo.messages.getString("ok"));
		UIManager.put("ColorChooser.resetText", Logo.messages
				.getString("restaurer"));
		UIManager.put("ColorChooser.cancelText", Logo.messages
				.getString("annuler"));
		UIManager.put("ColorChooser.previewText", Logo.messages
				.getString("apercu"));

		/*
		 * UIManager.put("Menu.font", Config.police);
		 * UIManager.put("MenuItem.font", Config.police);
		 * 
		 * UIManager.put("Label.font",new
		 * javax.swing.plaf.FontUIResource(Config.police));
		 * UIManager.put("Button.font", new
		 * javax.swing.plaf.FontUIResource(Config.police));
		 * UIManager.put("OptionPane.font", new
		 * javax.swing.plaf.FontUIResource(Config.police));
		 * 
		 * UIManager.put("TextArea.font", Config.police);
		 * UIManager.put("ComboBox.font", Config.police);
		 * UIManager.put("List.font", Config.police);
		 * UIManager.put("CheckBox.font", Config.police);
		 * UIManager.put("TabbedPane.font", new
		 * javax.swing.plaf.FontUIResource(Config.police));
		 * UIManager.put("TabbedPane.Font", new
		 * javax.swing.plaf.FontUIResource(Config.police));
		 * UIManager.put("TabbedPane.tabFont", new
		 * javax.swing.plaf.FontUIResource(Config.police));
		 */
		UIManager.put("ColorChooser.font", new javax.swing.plaf.FontUIResource(
				Config.police));
		//		System.out.println(UIManager.get("ColorChooser.font"));
		// On change la police et le texte de tous les composants texte
		UIManager.put("OptionPane.buttonFont", Config.police);
		commande.setFont(Config.police);
		jMenuFile.setFont(Config.police);
		jMenuFileEnregistrer_sous.setFont(Config.police);
		jMenuFileNouveau.setFont(Config.police);
		jMenuHelp.setFont(Config.police);
		jMenuHelpLicence.setFont(Config.police);
		jMenuHelpAbout.setFont(Config.police);
		jMenuHelplicencefrancais.setFont(Config.police);
		jLabel1.setFont(Config.police);
		jMenuEdition.setFont(Config.police);
		jMenuEditionCopier.setFont(Config.police);
		jMenuEditionColler.setFont(Config.police);
		jMenuEditionCouper.setFont(Config.police);
		jMenuEditionSelect.setFont(Config.police);
		jMenuFileOuvrir.setFont(Config.police);
		jMenuFileEnregistrer.setFont(Config.police);
		jMenuFileQuitter.setFont(Config.police);
		jMenuCapture.setFont(Config.police);
		jMenuZoneTexte.setFont(Config.police);
		jMenuZoneTexteItem.setFont(Config.police);
		jMenuImageCopy.setFont(Config.police);
		jMenuImageSave.setFont(Config.police);
		jMenuImagePrint.setFont(Config.police);
		jMenuOptions.setFont(Config.police);
		jMenuOptionsDemarrage.setFont(Config.police);
		jMenuOptionsCouleurCrayon.setFont(Config.police);
		jMenuOptionsCouleurFond.setFont(Config.police);
		jMenuOptionsPreference.setFont(Config.police);
		jMenuOptionsTraduction.setFont(Config.police);
		jMenuOptionsEraser.setFont(Config.police);
		
		this.panneauHistorique1.changeFont(Config.police);
		editeur.setEditorFont(Config.police);

		jMenuFile.setText(Logo.messages.getString("fichier"));
		jMenuFileEnregistrer_sous.setText(Logo.messages
				.getString("enregistrer_sous")+"...");
		jMenuFileNouveau.setText(Logo.messages.getString("nouveau"));
		jMenuHelp.setText(Logo.messages.getString("aide"));
		jMenuHelpLicence.setText(Logo.messages.getString("licence"));
		jMenuHelpAbout.setText(Logo.messages.getString("a_propos"));
		jMenuHelplicencefrancais.setText(Logo.messages
				.getString("licence_francais"));
		jLabel1.setText(Logo.messages.getString("commande") + "  ");
		jMenuEdition.setText(Logo.messages.getString("edition"));
		jMenuEditionCopier.setText(Logo.messages.getString("copier"));
		jMenuEditionColler.setText(Logo.messages.getString("coller"));
		jMenuEditionCouper.setText(Logo.messages.getString("couper"));
		jMenuEditionSelect.setText(Logo.messages.getString("tout_selectionner"));
		jMenuFileOuvrir.setText(Logo.messages.getString("ouvrir")+"...");
		jMenuFileEnregistrer.setText(Logo.messages.getString("enregistrer"));
		jMenuFileQuitter.setText(Logo.messages.getString("quitter"));
		jMenuCapture.setText(Logo.messages.getString("captureimage"));
		jMenuZoneTexte.setText(Logo.messages.getString("zone_de_texte"));
		jMenuZoneTexteItem.setText(Logo.messages.getString("enregistrer_rtf"));		
		jMenuImageCopy.setText(Logo.messages.getString("copier_image"));
		jMenuImageSave.setText(Logo.messages.getString("enregistreimage"));
		jMenuImagePrint.setText(Logo.messages.getString("imprimer"));
		jMenuOptions.setText(Logo.messages.getString("outils"));
		jMenuOptionsDemarrage.setText(Logo.messages
				.getString("definir_demarrage"));
		jMenuOptionsCouleurCrayon.setText(Logo.messages
				.getString("couleurcrayon"));
		jMenuOptionsCouleurFond.setText(Logo.messages.getString("couleurfond"));
		jMenuOptionsTraduction.setText(Logo.messages
				.getString("traduire_procedure"));
		jMenuOptionsEraser.setText(Logo.messages
				.getString("procedure_eraser"));
		jMenuOptionsPreference.setText(Logo.messages.getString("preferences"));
		
		panneauHistorique1.updateText();
		
		jpop.setText();
		
		//		JColorChooser jc=new JColorChooser();
		//		affiche_nom(jc);
	}

	/*
	 * void affiche_nom(Component jc){ jc.setFont(Config.police);
	 * System.out.println(jc.getName()); if (jc instanceof Container){ int
	 * nb=((Container)jc).getComponentCount(); for (int i=0;i <nb;i++){
	 * Component jcbis=((Container)jc).getComponent(i);
	 * System.out.println(jcbis.getFont()); affiche_nom(jcbis); } } }
	 */

	/** Remplacé, ainsi nous pouvons sortir quand la fenêtre est fermée */
	protected void processWindowEvent(WindowEvent e) {
		if (e.getID() == WindowEvent.WINDOW_CLOSING) {
			closeWindow();
		} else {
			super.processWindowEvent(e);
		}
	}
	protected void closeWindow(){
    	setVisible(true);
    	String message=Logo.messages.getString("quitter?");
		JTextArea jt=new JTextArea(message);
		jt.setEditable(false);
		jt.setBackground(new Color(255,255,177));
		jt.setFont(Config.police);
		ImageIcon icone=new ImageIcon(Utils.class.getResource("icone.png"));
		String[] choix={Logo.messages.getString("ok"),Logo.messages.getString("annuler")};
		int retval=JOptionPane.showOptionDialog(this,
				jt,Logo.messages.getString("quitter"),JOptionPane.OK_CANCEL_OPTION,JOptionPane.QUESTION_MESSAGE,icone,choix,choix[0]);	
      if (retval==JOptionPane.OK_OPTION){
      	// On écrit le fichier de configuration
        ecris_config();
        System.exit(0);
      }

		
	}

	// Ecrire le fichier de configuration lorsque l'on quitte XLOGO
	private void ecris_config() {
		try{
	        FileOutputStream f = new FileOutputStream(System.getProperty("user.home")+File.separator+".xlogo");
	        BufferedOutputStream b = new BufferedOutputStream(f);	
			OutputStreamWriter osw = new  OutputStreamWriter(b,  "UTF8");		
			StringBuffer sb =new StringBuffer(); 
			int eff = 0;
			if (Config.eraseImage)
				eff = 1;
			else
				eff = 0;
			sb.append("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
			sb.append("<xlogo>\n");
			sb.append("\t<lang value=\"");
			sb.append(Config.langage);
			sb.append("\"/>\n");
			sb.append("\t<speed value=\"");
			sb.append(Config.turtleSpeed);
			sb.append("\"/>\n");
			sb.append("\t<turtle_shape value=\"");
			sb.append(Config.activeTurtle);
			sb.append("\"/>\n");
			sb.append("\t<max_number_turtle value=\"");
			sb.append(Config.maxTurtles);
			sb.append("\"/>\n");
			sb.append("\t<pen_shape value=\"");
			sb.append(Config.penShape);
			sb.append("\"/>\n");
			sb.append("\t<cleanscreen_leaving_editor value=\"");
			sb.append(eff);
			sb.append("\"/>\n");
			sb.append("\t<pen_width_max value=\"");
			sb.append(Config.maxPenWidth);
			sb.append("\"/>\n");
			sb.append("\t<pen_color value=\"");
			sb.append(Config.pencolor.getRGB());
			sb.append("\"/>\n");
			sb.append("\t<screen_color value=\"");
			sb.append(Config.screencolor.getRGB());
			sb.append("\"/>\n");
			sb.append("\t<default_directory value=\"");
			sb.append(Utils.SortieTexte(Config.defaultFolder));
			sb.append("\"/>\n");
			sb.append("\t<start_command value=\"");
			sb.append(Utils.specialCharacterXML(Config.a_executer));
			sb.append("\"/>\n");
			sb.append("\t<font name=\"");
			sb.append(Config.police.getName());
			sb.append("\" size=\"");
			sb.append(Config.police.getSize());
			sb.append("\"/>\n");
			sb.append("\t<width value=\"");
			sb.append(Config.imageWidth);
			sb.append("\"/>\n");
			sb.append("\t<height value=\"");
			sb.append(Config.imageHeight);
			sb.append("\"/>\n");
			sb.append("\t<memory value=\"");
			sb.append(Config.tmp_memoire);
			sb.append("\"/>\n");
			sb.append("\t<quality value=\"");
			sb.append(Config.quality);
			sb.append("\"/>\n");
			sb.append("\t<looknfeel value=\"");
			sb.append(Config.looknfeel);
			sb.append("\"/>\n");
			sb.append("\t<syntax_highlighting\n\t\tboolean=\"");
			sb.append(Config.COLOR_ENABLED);
			sb.append("\"\n\t\tcolor_commentaire=\"");
			sb.append(Config.coloration_commentaire);
			sb.append("\"\n\t\tcolor_operand=\"");
			sb.append(Config.coloration_operande);
			sb.append("\"\n\t\tcolor_parenthesis=\"");
			sb.append(Config.coloration_parenthese);
			sb.append("\"\n\t\tcolor_primitive=\"");
			sb.append(Config.coloration_primitive);
			sb.append("\"\n\t\tstyle_commentaire=\"");
			sb.append(Config.style_commentaire);
			sb.append("\"\n\t\tstyle_operand=\"");
			sb.append(Config.style_operande);
			sb.append("\"\n\t\tstyle_parenthesis=\"");
			sb.append(Config.style_parenthese);
			sb.append("\"\n\t\tstyle_primitive=\"");
			sb.append(Config.style_primitive);
			sb.append("\">\n\t</syntax_highlighting>\n\t<startup_files\n");
			int i=0;
			while (!Config.path.isEmpty()) {
				String att="\t\tfile"+i+"=\"";
				sb.append(att);
				sb.append(Config.path.pop().toString());
				sb.append("\"\n");
				i++;
			}
			sb.append(">\n\t</startup_files>\n");
			sb.append("</xlogo>");
			osw.write(new String(sb));
		//	System.out.println(sb);
			osw.close();
			b.close();
			f.close();
		}
		catch(IOException e){System.out.println("write error");}
/*		try {
			FileWriter file = new FileWriter(System.getProperty("user.home")
					+ File.separator + ".xlogo");
			BufferedWriter bfile = new BufferedWriter(file);
			int eff = 0;
			if (Logo.effacer_dessin)
				eff = 1;
			else
				eff = 0;
			String st = "# langue\n" + Config.langage + "\n# vitesse\n"
					+ Logo.vitesse + "\n# tortue choisie\n"
					+ Logo.tortue_choisie + "\n# nb max de tortues\n"
					+ Logo.tortues + "\n# forme crayon\n" + Logo.forme_crayon
					+ "\n# effacer dessin en quittant editeur\n" + eff
					+ "\n# epaisseur max crayon\n" + Logo.epaisseur_max
					+ "\n# repertoire par defaut\n" + Utils.SortieTexte(Logo.repertoire_defaut)
					+ "\n# a executer au demarrage\n" + Config.a_executer
					+ "\n# police\n" + Config.police.getName() + "\n"
					+ Config.police.getSize() + "\n# largeur\n"
					+ Logo.largeur_dessin + "\n# hauteur\n"
					+ Logo.hauteur_dessin + "\n# memoire\n"+Config.tmp_memoire
					+"\n# qualite\n"+Logo.quality+"\n# coloration\n"+Config.COLOR_ENABLED+" "+Config.coloration_commentaire+" "+
					Config.coloration_operande+" "+Config.coloration_parenthese+" "+Config.coloration_primitive+" "
					+Config.style_commentaire+" "+Config.style_operande+" "+Config.style_parenthese+" "+Config.style_primitive
					+"\n# fichiers de demarrage\n";
			while (!Config.path.isEmpty()) {
				st += Config.path.pop().toString() + "\n";
			}
			st += "# fin";

			bfile.write(st);
			bfile.close();
		} catch (Exception ex) {
		}*/

	}

	// Ce qu'il se passe en validant dans la zone de texte
	public void commande_actionPerformed() {
	//	System.out.println("commandeTotal :"+Runtime.getRuntime().totalMemory()/1024/1024+" Free "+Runtime.getRuntime().freeMemory()/1024/1024);
		// Si une parenthese était sélectionnée, on désactive la décoloration
		commande.setActive(false);
		if (stop)
			stop = false;
		String texte = commande.getText();
		if (!texte.equals("") && commande.isEditable()) {
			if (touche.tape) {
				touche.tape = false;
				pile_historique.pop();
			}
			if (pile_historique.size() > 49)
				pile_historique.remove(0);
			pile_historique.push(texte); // On rajoute ce texte à l'historique
			index_historique = pile_historique.size(); //on réajuste l'index de
			// l'historique

			panneauHistorique1.ecris("normal", texte + "\n");
			
			// On enlève les éventuels commentaires
			int a =texte.indexOf("#");
			while (a!=-1){
				if (a==0) {texte="";break;}
				else if (!texte.substring(a-1,a).equals("\\")){
					texte=texte.substring(0,a);
					break;
				} 
				a =texte.indexOf("#",a+1);
			}
			affichage = new Affichage(this, Utils.decoupe(texte));
			affichage.start();
			// On efface la ligne de commande
			SwingUtilities.invokeLater(new Runnable(){
				public void run(){
					commande.setText("");					
				}
			});

		}
	}

	public Sound_Player getSon(){
		return son;
	}
	public void close_Preference(){
		pf=null;
	}
	public void close_DialogueImage(){
		diag=null;
	}
	// change language for the interface
	// change la langue de l'interface
	public void changeLanguage(int id){
		Config.langage = id;
		Logo.generateLanguage(id);
		setText();
		kernel.buildPrimitiveTreemap(id);
		editeur = new Editor(this);
		panneauHistorique1.changeLanguage();
	}
	// Change font for the interface
	// Change la police de l'interface
	public void changeFont(Font font,int size){
		Config.police = font;
		setText();
		panneauHistorique1.getDsd().change_police_interface(font, size);
	}
	public void resizeDrawingZone(){
		// resize the drawing image
		Ardoise.dessin=new BufferedImage(Config.imageWidth,Config.imageHeight,BufferedImage.TYPE_INT_RGB);
		 // 	System.out.println("Total :"+Runtime.getRuntime().totalMemory()/1024+" max "+Runtime.getRuntime().maxMemory()/1024+" Free "+Runtime.getRuntime().freeMemory()/1024);
		MediaTracker tracker=new MediaTracker(ardoise1);
		tracker.addImage(Ardoise.dessin,0);
		try{tracker.waitForID(0);}
		catch(InterruptedException e){}
	//    ardoise1.getGraphics().drawImage(Ardoise.dessin,0,0,ardoise1);
	    ardoise1.setPreferredSize(new Dimension(Config.imageWidth,Config.imageHeight));
	    ardoise1.revalidate();
	    ardoise1.repaint();
	    kernel.updateGraphics();
/*	    Dimension d=jScrollPane1.getViewport().getViewRect().getSize();
	    Point p=new Point(Math.abs(Config.imageWidth/2-d.width/2),Math.abs(Config.imageHeight/2-d.height/2));
	    jScrollPane1.getViewport().setViewPosition(p);  	
*/
	}
	public void changeLookNFeel() throws Exception {
		SwingUtilities.updateComponentTreeUI(this);
		SwingUtilities.updateComponentTreeUI(editeur);
	}
	/* Change Syntax Highlighting for the editor, 
	 * the command line and the History zone
	 * */ 
	public void changeSyntaxHighlightingStyle(int c_comment,int sty_comment,int c_primitive,int sty_primitive,
			int c_parenthese, int sty_parenthese, int c_operande, int sty_operande){
		editeur.initStyles(Config.coloration_commentaire,Config.style_commentaire,Config.coloration_primitive,Config.style_primitive,
				Config.coloration_parenthese,Config.style_parenthese,Config.coloration_operande,Config.style_operande);
		commande.initStyles(Config.coloration_commentaire,Config.style_commentaire,Config.coloration_primitive,Config.style_primitive,
				Config.coloration_parenthese,Config.style_parenthese,Config.coloration_operande,Config.style_operande);
		panneauHistorique1.getDsd().initStyles(Config.coloration_commentaire,Config.style_commentaire,Config.coloration_primitive,Config.style_primitive,
				Config.coloration_parenthese,Config.style_parenthese,Config.coloration_operande,Config.style_operande);
	}
  // Enable or disable Syntax Highlighting 
	public void setColoration(boolean b){
		editeur.setColoration(b);
		panneauHistorique1.setColoration(b);
		commande.setColoration(b);
	}
	public void resizeCommande(){
		commande.setPreferredSize(new Dimension(300,Config.police.getSize()*18/10));
	}

	public Ardoise getArdoise(){
		return ardoise1;
	}  
	public void ecris(String sty,String txt){
		panneauHistorique1.ecris(sty,txt);
	}
	public void focus_Commande(){
		commande.requestFocus();
	}
	public void setEnabled_Record(boolean b){
		jMenuFileEnregistrer.setEnabled(true);
	}
	public void setEnabled_New(boolean b){
		jMenuFileNouveau.setEnabled(b);
	}
	public boolean isEnabled_new(){
		return jMenuFileNouveau.isEnabled();
	}
	public boolean affichage_Runnning(){
		return null!=affichage;
	}
	public boolean commande_isEditable(){
		return commande.isEditable();
	}
	public void setCommandText(String txt){
		commande.setText(txt);
	}
	public HistoryPanel getHistoryPanel(){
		return panneauHistorique1;
	}
	// Enable or disable the command line
	public void setCommandLine(boolean b){
		if (b) {
			if (SwingUtilities.isEventDispatchThread()){
				commande.setEditable(true);
				commande.setBackground(Color.WHITE);
			}
			else {
				SwingUtilities.invokeLater(new Runnable() {
					   public void run() {
							commande.setEditable(true);				
						   commande.requestFocus();
							commande.setBackground(Color.WHITE);
					   }
					  });
				
				
			}
		}
		else {
			commande.setEditable(false); 
			commande.setBackground(new Color(250, 232, 217));
		}
	}
	protected void copy(){
		commande.copy();
	}
	protected void cut(){
		commande.cut();
	}
	protected void paste(){
		commande.paste();
	}
	protected void select_all(){
		commande.selectAll();
	}
	 protected void genere_primitive(){
	 	kernel.initPrimitive();
	    // Générer les procédures du path
	    Stack tampon=new Stack();
		editeur.setAffichable(false);
	    int compteur=Config.path.size();

	    for (int i=0;i<compteur;i++){
	      String txt="";
			if (Config.path.get(i).equals("#####")) editeur.setAffichable(true); //on a terminé avec les fichiers de démarrage
	    else {
			try{
				txt=Utils.readLogoFile(Config.path.get(i).toString());
			  	if (!editeur.getAffichable()) tampon.push(Config.path.get(i).toString()); 
			  }
			  catch(IOException e2){System.out.println("Problème de lecture du fichier");}
			  catch(Exception e3){}
			editeur.setEditorText(txt);
			try {
			  editeur.analyseprocedure();
			}
			catch(Exception e3){
				System.out.println(e3.toString());
			}
			editeur.setVisible(false);
		    }
	    }
	  Config.path=(Stack)tampon.clone(); //On ne garde dans le path que les fichiers de démarrage
	  editeur.setAffichable(true);
	  editeur.setEditorText("");
	 }	
	 public void setCar(int i){
	 	touche.setCar(i);
	 } 
	 public int getCar(){
	 	return touche.getCar();
	 }
	 
	 protected void prefOpen(){
	 	if (null==pf) {
			pf=new Preference(this);
			pf.setBounds(100,100,600,580);
			pf.setVisible(true);
		}
		else pf.requestFocus();
	 } 
	 protected void demOpen(){
	  	if (null==dem||!dem.isVisible()){
	        dem=new Demarrage(this);
	        dem.setBounds(100,100,400,250);
	        dem.setVisible(true);
	      }
	      else dem.requestFocus();
	 }
	 protected void traducOpen(){
		if (null==traduc||!traduc.isVisible()){
		      traduc=new Traduc();
		      traduc.setVisible(true);
		    }
		    else {
		    	traduc.setVisible(false);
		    	traduc.setVisible(true);
	      	}
	 }
	 protected void eraserOpen(){
	 	if (null==eraser||!eraser.isVisible()){
  			eraser=new ProcedureEraser(this);
  			eraser.setBounds(100,100,300,200);
  			eraser.setVisible(true);
  	}
  	else eraser.requestFocus();
	}
	public void eraserUpdate(){
	 	if (null!=eraser&&eraser.isVisible()){
	 		eraser.dispose();
  			eraser=new ProcedureEraser(this);
  			eraser.setBounds(100,100,300,200);
  			eraser.setVisible(true);
	 	}
	}
	 protected void diagOpen(String st){
	    if (null==diag){
	        diag = new Dialogueimage(this, st);
	        diag.setBounds(getX(), getY(), getWidth() / 2, getHeight() / 4);
	        diag.setVisible(true);
	      }
	    else diag.requestFocus();
	 }
	 public Kernel getKernel(){
	 	return kernel;
	 }
	class Touche extends KeyAdapter {
		int car = -1;

		private boolean tape = false;
		public void setCar(int i){
			car=i;
		}
		public int getCar(){
		return car;	
		}
		public void keyPressed(KeyEvent e) {
			int ch = e.getKeyChar();
			int code =e.getKeyCode();
			if (commande.isEditable()) {
				if (code == KeyEvent.VK_UP) {
					if (index_historique > 0) {
						if (index_historique == pile_historique.size()) {
							tape = true;
							pile_historique.push(commande.getText());
						}
						index_historique--;
						commande.setText(pile_historique.get(index_historique)
								.toString());
					} else
						index_historique = 0;
				}
				else if (code == KeyEvent.VK_DOWN) {
					if (index_historique < pile_historique.size() - 1) {
						index_historique++;
						commande.setText(pile_historique.get(index_historique)
								.toString());
					} else
						index_historique = pile_historique.size() - 1;
				}
			} else {
				if (ch!=65535)	car = ch;
				else car=-code;
			}
		}
	}

	class PopupListener extends MouseAdapter {

		public void mousePressed(MouseEvent e) {
			maybeShowPopup(e);
		}

		public void mouseReleased(MouseEvent e) {
			maybeShowPopup(e);
		}

		private void maybeShowPopup(MouseEvent e) {
			if (e.isPopupTrigger()) {
				jpop.show(e.getComponent(), e.getX(), e.getY());
			}
		}
	}
}