/**
 * Title : XLogo Description : XLogo is an interpreter for the Logo programming
 * language
 * 
 * @author Loïc Le Coq
 */
package xlogo.kernel;

import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Stack;
import java.util.StringTokenizer;
import java.util.TreeMap;
import java.math.BigDecimal;
import xlogo.Config;
import xlogo.kernel.LoopProperties;
import xlogo.utils.Utils;
import xlogo.utils.myException;
import xlogo.Application;
import java.io.*;
import xlogo.Logo;
public class Primitive {
	//  float taille_crayon=(float)0;
	private Application cadre;
	private Kernel kernel;
	protected static final int PRIMITIVE_NUMBER = 230;

	protected int[] parametres=new int[PRIMITIVE_NUMBER];
	
	// Treemap for primitives (better efficiency in searching)
	public static TreeMap primitives = new TreeMap(); 


	public static Stack stackLoop = new Stack();

	public Primitive() {
	}

	public Primitive(Application cadre) {
		this.cadre = cadre;
		this.kernel=cadre.getKernel();
		//build treemap for primitives
		buildPrimitiveTreemap(Config.langage);
	}

	//Exécution des primitives
	public void buildPrimitiveTreemap(int id) {
	//	exportCSV();
		primitives = new TreeMap();
		Locale locale = Logo.generateLocale(Config.langage);
		ResourceBundle prim = ResourceBundle.getBundle("primitives", locale);
		try{
			BufferedReader bfr=new BufferedReader(
					new InputStreamReader(Primitive.class.getResourceAsStream("genericPrimitive")));
			int i=0;
			while(bfr.ready()){
				String line=bfr.readLine();
				StringTokenizer cut=new StringTokenizer(line); 
				String cle = cut.nextToken();
				parametres[i]=Integer.parseInt(cut.nextToken());
				if (i == 39)
					primitives.put("\n", "39");
				else if (i==212)
					primitives.put("\\x","212");
				else {
					if (cle.length() != 1) {
						cle = prim.getString(cle);
					}
					StringTokenizer st = new StringTokenizer(cle);
					int compteur = 0;
					while (st.hasMoreTokens()) {
						primitives.put(st.nextToken(), String.valueOf(i
								+ Primitive.PRIMITIVE_NUMBER * compteur));
						compteur++;
					}
				}
				i++;
			}
//			System.out.println(i+ " primitives");
		}
		catch(IOException e){System.out.println("Impossible de lire le fichier d'initialisation des primitives");}
	}
	// primitive distance
	protected double distance(String liste, String nom) throws myException {
		double y = 1.0;
		double x = 1.0;
		StringTokenizer st = new StringTokenizer(liste);
		try {
			if (!st.hasMoreTokens())
				throw new myException(cadre, nom + " "
						+ Logo.messages.getString("n_aime_pas") + liste
						+ Logo.messages.getString("comme_parametre"));
			String li = st.nextToken().toString();
			if (li.equals("-")) {
				if (st.hasMoreTokens())
					li = st.nextToken().toString();
				x = -1;
			}
			x = x * Double.parseDouble(li) + Config.imageWidth / 2;
			if (!st.hasMoreTokens())
				throw new myException(cadre, nom + " "
						+ Logo.messages.getString("n_aime_pas") + liste
						+ Logo.messages.getString("comme_parametre"));
			li = st.nextToken().toString();
			if (li.equals("-")) {
				if (st.hasMoreTokens())
					li = st.nextToken().toString();
				y = -1;
			}
			y = Config.imageHeight / 2 - Double.parseDouble(li) * y;
		} catch (NumberFormatException e) {
			throw new myException(cadre, nom + " "
					+ Logo.messages.getString("n_aime_pas") + liste
					+ Logo.messages.getString("comme_parametre"));
		}
		if (st.hasMoreTokens())
			throw new myException(cadre, nom + " "
					+ Logo.messages.getString("n_aime_pas") + liste
					+ Logo.messages.getString("comme_parametre"));
		Interprete.operande = true;
		if (nom.equals("distance")) {
			double distance = Math.sqrt(Math.pow(kernel.getActiveTurtle().corX - x, 2)
					+ Math.pow(kernel.getActiveTurtle().corY - y, 2));
			return distance;
		} else {
			double angle;
			if (kernel.getActiveTurtle().corY == y) {
				if (x > kernel.getActiveTurtle().corX)
					angle = 90;
				else if (x == kernel.getActiveTurtle().corX)
					angle = 0;
				else
					angle = 270;
			} else if (kernel.getActiveTurtle().corX == x) {
				if (kernel.getActiveTurtle().corY > y)
					angle = 0;
				else
					angle = 180;
			} else {
				angle = Math.toDegrees(Math.atan(Math
						.abs(x - kernel.getActiveTurtle().corX)
						/ Math.abs(kernel.getActiveTurtle().corY - y)));
				if (x > kernel.getActiveTurtle().corX && y > kernel.getActiveTurtle().corY)
					angle = 180 - angle; // 2eme quadrant
				else if (x < kernel.getActiveTurtle().corX && y > kernel.getActiveTurtle().corY)
					angle = 180 + angle; // 3eme quadrant
				else if (x < kernel.getActiveTurtle().corX && y < kernel.getActiveTurtle().corY)
					angle = 360 - angle; // 4eme quadrant
			}
			return angle;
		}
	}
	// primitive repeat
	protected void repete(int i, String st) {
		if (i > 0) {
			st = new String(Utils.decoupe(st));
			LoopProperties bp = new LoopProperties(LaunchPrimitive.un,
					new BigDecimal(i), LaunchPrimitive.un, st, "repete");
			stackLoop.push(bp);
			Interprete.instruction.insert(0, st + "\\ ");
		}
	}
	// primitive if
	protected void si(boolean b, String li, String li2) {
		if (b) {
			Interprete.instruction.insert(0, li);
		} else if (null != li2) {
			Interprete.instruction.insert(0, li2);
		}
	}
	// internal primitive for loop while.
	protected void whilesi(boolean b, String li) {
		if (b) {
			Interprete.instruction.insert(0, li
					+ ((LoopProperties) Primitive.stackLoop.peek())
							.getInstr());
		} else {
			try {
				Interprete.tueniveau("\\", cadre);
			} catch (myException e) {
			}
		}
	}
	// primitive stop
	protected void stop() {
		Interprete.operande = false;
		try {
			Interprete.tueniveau("\\", cadre);
		} catch (myException e) {
		}
	}
	// primitive output
	protected void retourne(String val) throws myException {
		Interprete.calcul.push(val);
		Interprete.operande = true;
		if (Kernel.mode_trace) {
			StringBuffer buffer = new StringBuffer();
			for (int i = 0; i < Interprete.en_cours.size() - 1; i++)
				buffer.append("  ");
			buffer.append(Interprete.en_cours.peek());
			buffer
					.append(" " + Logo.messages.getString("retourne") + " "
							+ val);
			cadre.ecris("normal", Utils.SortieTexte(buffer.toString()) + "\n");
		}
		if ((!Interprete.nom.isEmpty())
				&& Interprete.nom.peek().toString().equals("\n")) {
			try {
				Interprete.tueniveau("\n", cadre);
			} catch (myException e) {
			}
			Interprete.nom.pop();
		} else if (!Interprete.nom.isEmpty())
			throw new myException(cadre, Logo.messages.getString("retourne")
					+ " " + Logo.messages.getString("ne_renvoie_pas") + " "
					+ Interprete.nom.peek().toString());
		else
			throw new myException(cadre, Logo.messages
					.getString("erreur_retourne"));
	}
/*	private void exportCSV(){
		StringBuffer sb=new StringBuffer();
		Locale locale=new Locale("fr","FR");
		ResourceBundle prim_fr = ResourceBundle.getBundle("primitives", locale);	
		locale=new Locale("en","US");
		ResourceBundle prim_en = ResourceBundle.getBundle("primitives", locale);
		locale=new Locale("ar","MA");
		ResourceBundle prim_ar = ResourceBundle.getBundle("primitives", locale);
      	locale=new Locale("es","SP");
      	ResourceBundle prim_es = ResourceBundle.getBundle("primitives", locale);
        locale=new Locale("pt","BR");
        ResourceBundle prim_pt = ResourceBundle.getBundle("primitives", locale);
       	locale=new Locale("eo","EO");
       	ResourceBundle prim_eo = ResourceBundle.getBundle("primitives", locale);
       	locale=new Locale("de","DE");
       	ResourceBundle prim_de = ResourceBundle.getBundle("primitives", locale);
        //locale=new Locale("nl","NL");
       	ResourceBundle[] prim={prim_fr,prim_en,prim_es,prim_pt,prim_ar,prim_eo,prim_de};
		try{
			BufferedReader bfr=new BufferedReader(
					new InputStreamReader(Primitive.class.getResourceAsStream("genericPrimitive")));
			int i=0;
			while(bfr.ready()){
				String line=bfr.readLine();
				StringTokenizer cut=new StringTokenizer(line); 
				String cle = cut.nextToken();
				parametres[i]=Integer.parseInt(cut.nextToken());
				if (i == 39)
					primitives.put("\n", "39");
				else if (i==212)
					primitives.put("\\x","212");
				else {
					System.out.println(i);
					for (int j=0;j<prim.length;j++){
						String txt=cle;
						if (cle.length() != 1) {
							txt= prim[j].getString(cle);
						}
						sb.append(txt);
						sb.append(";");							
					}
					sb.append("\n");
				}
				i++;
			}
			Utils.writeLogoFile("/home/loic/table", new String(sb));
		}
		catch(Exception e){e.printStackTrace();}
	}*/
}