/**
 * Title :        XLogo
 * Description :  XLogo is an interpreter for the Logo 
 * 						programming language
 * @author Loïc Le Coq
 */
package xlogo.kernel;

import java.util.Stack;
import xlogo.Logo;
import xlogo.Application;
import java.io.BufferedReader;
import java.io.StringReader;
import java.io.IOException;
public class Workspace {
	protected  Stack globale;  // Stack with the name of all defined variablesPile contenant les noms des variables globales
	protected  Stack valeur; // Stack with their values
	private Stack procedureList; // stack with all procedures defined by the user
	public Workspace() {
		globale= new Stack();
		valeur= new Stack();
		procedureList=new Stack();

	}
	public int searchVariable(String st){
		return globale.search(st);
	}
	public void deleteAllVariables(){
		globale = new Stack();
		valeur = new Stack();
	}
	public void deleteAllProcedures(){
		for (int i = procedureList.size()-1; i > -1; i--) {
			  Procedure procedure = getProcedure(i);
			  if (procedure.affichable == true)
				deleteProcedure(i);
			}
	}
	public Procedure procedureListPeek(){
		return  (Procedure)(procedureList.peek());
	}
	public void procedureListPop(){
		procedureList.pop();
	}
	public void procedureListPush(Procedure pr){
		procedureList.push(pr);
	}
	public void setProcedureList(int id, Procedure pr){
		procedureList.set(id,pr);
	}
	public Procedure getProcedure(int index){
		return (Procedure)(procedureList.get(index)); 	
	}
	public int getNumberOfProcedure(){
		return procedureList.size();
	}
	public void deleteProcedure(int index){
		procedureList.remove(index);
	}

	public void deleteVariable(String st){
		int index = globale.search(st);
		if (index != -1) {
			globale.remove(index - 1);
			valeur.remove(index - 1);
		}
	}
	public String toString(){
		StringBuffer sb=new StringBuffer();
		for (int i=0;i<globale.size();i++){
			sb.append("-");
			sb.append(globale.get(i));
			sb.append("\n");
			sb.append(valeur.get(i));
			sb.append("\n");
		}
		for(int i=0;i<getNumberOfProcedure();i++){
	        Procedure procedure =getProcedure(i);
	        sb.append(Logo.messages.getString("pour") + " " + procedure.name);
	        for (int j = 0; j < procedure.nbparametre; j++) {
	        	sb.append(" :" +procedure.variable.get(j).toString());
	        }
	        sb.append("\n");
	        sb.append(procedure.instruction);
	        sb.append(Logo.messages.getString("fin"));
	        sb.append("\n\n");
	      }
		return (new String(sb));
	}
	public void setWorkspace(Application app,String wp){
		globale= new Stack();
		valeur= new Stack();
		procedureList=new Stack();

		StringReader sr=new StringReader(wp);
		BufferedReader bfr=new BufferedReader(sr);
		try{
			String input="";
			boolean name=false;
			while((input=bfr.readLine())!=null){
				if (!input.startsWith(Logo.messages.getString("pour"))){
					if (!name)	{
						globale.push(input.substring(1));
						name=true;
					}
					else {
						name=false;
						valeur.push(input);
					}
				}
				else break;
			}
			StringBuffer sb=new StringBuffer();
			if (null!=input){
				sb.append(input);
				sb.append("\n");
			}
			while((input=bfr.readLine())!=null){
				sb.append(input);
				sb.append("\n");
			}
			bfr.close();
			sr.close();
			try{
				app.editeur.setAffichable(false);
				app.editeur.setEditorText(new String(sb));
				app.editeur.analyseprocedure();
			}
			catch(Exception e){
			}
			
		}
		catch(IOException e){
		}
	}

}
